Character = Object:extend()

function Character:new(x,y,width,height)
	self.x = x
	self.y = y
	self.width = width
	self.height = height

	self.xVelocity = 150
	self.yVelocity = 0
	self.pathVelocity = 250

	self.jumpHeight = -300
	self.gravity = -900

	-- STATES
	-- direction -> idle, left, right
	self.direction = "idle"
	-- state -> on_ground, jumping, falling
	self.state = "on_ground"
	-- cocoonaSate -> cocoona, not_cocoona
	self.cocoonaState = "not_cocoona"

	-- WALL JUMPING STATE
	-- lastTouched -> left_wall, right_wall, nil
	self.lastTouched = nil

	self.jumpNo = 0

	tween = nil
end

function Character:movePlayer(xgoal,ygoal,dt)
		--Check collision
		local playerFilter = function(item, other)
			if self.cocoonaState == "not_cocoona" then
				if other.name == "plat" or other.name == "box_spawn" then
	  				return 'slide'
	  			end
				if other.name == "enemy1" then
	  			    return 'bounce'
	  			end
	  			if other.properties.collidable == false then 
	  				return nil
	  			end
	  			if other.name == "star_spawn" then
	  				return 'cross'
	  			end
	  		else
	  			if other.name == "enemy1" then
	  			    return 'cross'
	  			end
	  		end
  			return nil
		end

		local playerFilter2 = function(item, other)
			return nil
		end

		local goalX, goalY = xgoal, ygoal
		-- fixed jumping to different x issue
		if justPathed then
				local actualX, actualY, cols, len = world:move(self, goalX, goalY,playerFilter2)
			justPathed = false
		end

  		local actualX, actualY, cols, len = world:move(self, goalX, goalY, playerFilter)
  		self.x, self.y = math.floor(actualX), math.floor(actualY)

  		return cols
	end


function Character:update(dt)
	print(self.x, self.y)
	-- ACTIVATING COCOONA
	if love.keyboard.isDown("up") then
		-- SET COCOONA STATE
		self.cocoonaState = "cocoona"
		-- Initiate closestObj and distances
		local closestObj = path[1]
		local xDistance = self.x - path[1].x
		local yDistance = self.y - path[1].y

		-- PATH FINDING - TO BE IMPLEMENTED (USING A* ALG?)
		local findPath = function()
		end

		-- PATH FOLLOWING
		local followPath = function()
			-- Set closestObj to the closest path
			if xDistance < 0 then
				xDistance = -xDistance
			end
			if yDistance < 0 then
				yDistance = -yDistance
			end
			local xyDistance = xDistance + yDistance
			local index = 1
			for i = 1, #path do
				local xDistance = self.x - path[i].x
				if xDistance < 0 then
					xDistance = -xDistance
				end
				local yDistance = self.y - path[i].y
				if yDistance < 0 then
					yDistance = -yDistance
				end
				if xDistance + yDistance <= xyDistance and path[i].visited == false then
					xyDistance = xDistance + yDistance
					closestObj = path[i]
					index = i
				end
			end

			-- If we are on the next path square, set it to visited
			if self.x >= closestObj.x and self.y >= closestObj.y then
				closestObj.visited = true
				-- Fixes getting stuck at path[1] issue
				if index == 1 then
					closestObj = path[index + 1]
				end	
			end

			-- Move to next closest path square at speed pathVelocity
			local moveAlong = function(closestObject)
				-- local startX = self.x
				-- local startY = self.y
				-- local targetX = closestObject.x
				-- local targetY = closestObject.y
				-- local distance = (targetX - startX)^2 + (targetY - startY)^2
				-- tween = flux.to(self, distance/(self.pathVelocity*10), { x = closestObject.x, y = closestObject.y })
				-- dist2 - the distance to the closest object (in a straight line)
				local dist2= (closestObject.x - self.x)^2 + (closestObject.y - self.y)^2
				local obj_vx, obj_vy = 0,0
				local startPosX, startPosY = self.x,self.y
				local pv = self.pathVelocity
				-- Speed up for when falling - to look more realistic
				if closestObject.y > self.y then
					pv = self.pathVelocity - self.gravity * 5 * dt
				end
				if dist2 == 0 then
					-- avoid division by 0
				    obj_vx, obj_vy = 0, 0
				else
				    local dist = math.sqrt(dist2)
				    obj_vx = (closestObject.x - self.x) / dist * pv
				    obj_vy = (closestObject.y - self.y) / dist * pv
				end

		  		cols = self:movePlayer(self.x + obj_vx * dt, self.y + obj_vy * dt, dt)

		  		--Check collisions
		  		if #cols ~=0 then
		  			for i=1,#cols do
	  					local c = cols[i]
	  					if c.other.name == "enemy1" then
				  			local elayer = map.layers["EnemySprites"]
				  			-- if touch point is at top of enemy
				  			local x,y,w,h = world:getRect(c.other)				  			
					  		for i = 1, #elayer do
					  			local xx,yy = elayer[i].x,elayer[i].y
								if x == xx and y == yy then
									world:remove(c.other)
									table.remove(elayer,i)
									break
								end
							end
				  		end
	  				end
		  		end

		  		-- dist3 - distance from the start position to position now (in a straight line)
		  		local dist3 = (startPosX - self.x)^2 + (startPosY - self.y)^2
				if dist3 >= dist2 then
					print("3: ".. dist3)
					print("2: ".. dist2)
				-- Overshot - teleport to target instead.
				    self.x = closestObject.x
    				self.y = closestObject.y 
				    obj_vx = 0
				    obj_vy = 0
				end
			end
			-- print("closestObj: " .. closestObj.x,closestObj.y)
			-- print("self: ".. self.x,self.y)
			moveAlong(closestObj)
		end

		-- findPath()
		-- if on path then...
		followPath()

		justPathed = true
	else
		if tween then
			tween:stop()
		end
		-- SET cocoona STATE
		self.cocoonaState = "not_cocoona"
		-- if justPathed == true then
		-- 	-- JUMP
		-- 	state = "jumping"
		-- end

		-- NOT USING COCOON
		for i = 1, #path do
			path[i].visited = false
		end

		local newX, newY

		-- SET DIRECTION ON BUTTON PRESSED
		if love.keyboard.isDown("a") then
			self.direction = "left"
		elseif love.keyboard.isDown("d") then
			self.direction = "right"
		else
			self.direction = "idle"
		end

		-- SET STATE
		if (love.keyboard.isDown("w") and self.state == "on_ground") or justPathed then
			self.state = "jumping"
			self.jumpNo = 1
		end

		-- CHANGE X BASED ON DIRECTION
		if self.direction == "left" then
			newX = self.x - self.xVelocity * dt
		elseif self.direction == "right" then
			newX = self.x + self.xVelocity * dt
		elseif self.direction == "idle" then
			newX = self.x
		end

		-- CHANGE Y BASED ON STATE
		if self.state == "jumping" then
			self.yVelocity = self.jumpHeight
		elseif self.state == "falling" then
			self.yVelocity = self.yVelocity - self.gravity * dt
		elseif self.state == "on_ground" then
			self.yVelocity = 0
			self.jumpNo = 0
		end

		newY = self.y + self.yVelocity * dt

		-- Move player using bump.lua
		cols = self:movePlayer(newX,newY,dt)

		if #cols == 0 then
	  		self.state = "falling"
	  		self.lastTouched = nil
	  	end
	  	
		for i=1,#cols do
	  		local c = cols[i]
	 		-- IF COLLISION IS A PLATFORM
	  		if c.other.name == "plat" then
	  			local x,y,w,h = world:getRect(c.other)
	  			if c.touch.y < y then
	  				-- TOUCHING GROUND
	  				self.state = "on_ground"
	  				self.lastTouched = nil
	  				self.jumpNo = 0
	  			elseif c.touch.x <= x then
	  				-- TOUCHING RIGHT
					if self.lastTouched == "right_wall" and self.jumpNo == 1 then
						self.state = "falling"
					else
						self.state = "on_ground"
					end
					self.lastTouched = "right_wall"
				elseif c.touch.x >= x + w then
					-- TOUCHING LEFT
					if self.lastTouched == "left_wall" and self.jumpNo == 1 then
						self.state = "falling"
					else
						self.state = "on_ground"
					end
					self.lastTouched = "left_wall"
				end 
	  		end

	 		-- IF COLLISION IS AN ENEMY 
	  		if c.other.name == "enemy1" then
	  			local elayer = map.layers["EnemySprites"]
	  			-- if touch point is at top of enemy
	  			local x,y,w,h = world:getRect(c.other)
	  			if c.touch.y == y - h then
	  				self.state = "jumping"
	  				
		  			for i = 1, #elayer do
		  				local xx,yy = elayer[i].x,elayer[i].y
						if x == xx and y == yy then
							world:remove(c.other)
							table.remove(elayer,i)
							break
						end
					end
	  			else
					print ("DEAD")
					--love.event.quit()
				end
	  		end

	  		-- IF COLLISION IS A BOX
	  		if c.other.name == "box_spawn" then		
	  			local x,y,w,h = world:getRect(c.other)
	  			if c.touch.y < y then
	  				self.state = "on_ground"
				elseif c.touch.y == y + h then
					self.state = "on_ground"
					Boxes.changeColour(c.other)
				end
	  		end

	  		-- IF COLLISION IS A STAR
	  		if c.other.name == "star_spawn" then
	  			local x,y,w,h = world:getRect(c.other)
	  			world:remove(c.other)
	  			local sslayer = map.layers["StarSprites"]
	  			for i = 1, #map.layers["StarSprites"] do
					if x == sslayer[i].x and y == sslayer[i].y then
						table.remove(sslayer,i)
						break
					end
				end
	  		end
	  	end
	end
end

function Character:draw()
	if self.cocoonaState == "cocoona" then
		love.graphics.circle("fill", self.x, self.y + self.height/2, self.width/2)
	else
		love.graphics.rectangle("fill", self.x, self.y, self.width, self.height)
	end
end




