Game = Object:extend()
local sti = require "libraries.sti" 
local bump = require "libraries.bump"

function Game:new()
	map = sti("game/tilemap.lua", { "box2d" })
	local playerLayer = map:addCustomLayer("PlayerSprite",#map.layers + 2)
	local enemyLayer = map:addCustomLayer("EnemySprites",#map.layers + 2)
	local starLayer = map:addCustomLayer("StarSprites",#map.layers + 2)
	local boxLayer = map:addCustomLayer("BoxSprites",#map.layers + 2) 

	world = bump.newWorld()
	love.physics.setMeter(16)
	physicsWorld = love.physics.newWorld(0, 0)


	-- ADD OBJECTS TO COLLISION WORLD
	local addToBumpMap = function(objectName)
		for k, object in pairs(map.objects) do
			if object.name == objectName then 
				world:add(object,object.x,object.y,object.width,object.height)
			end
		end
	end

	-- SPAWN PLAYER AND ADD TO COLLISION WORLD
	local spawnPlayer = function(objectName)
		local player
		for k, object in pairs(map.objects) do
			if object.name == objectName then 
				player = object
				break
			end
		end
		playerLayer.player = Character(player.x,player.y,16,16)
    	playerLayer.player.name = player.name
		playerLayer.player.properties = player.properties
		world:add(playerLayer.player,player.x,player.y,16,16)

		map:removeLayer("player_spawn")
	end

	-- SPAWN ENEMIES AND ADD TO COLLISION WORLD
	local spawnEnemies = function(objectName)
		local enemy
		local i = 1
		for k, object in pairs(map.objects) do
			if object.name == objectName then 
				enemy = object
				enemyLayer[i] = Enemy(enemy.x,enemy.y,enemy.width,enemy.height)
				enemyLayer[i].name = enemy.name
				enemyLayer[i].properties = enemy.properties
				world:add(enemyLayer[i],enemy.x,enemy.y,enemy.width,enemy.height)

				i = i + 1
			end
		end
    	map:removeLayer("enemy_spawn")
	end

	-- SPAWN STARS AND ADD TO COLLISION WOLRD
	local spawnStars = function(objectName)
		local star
		local i = 1
		for k, object in pairs(map.objects) do
			if object.name == objectName then 
				star = object
				local sprite = love.graphics.newImage("assets/star.png")
			    starLayer[i] = {
			        sprite = sprite,
			        x      = star.x,
			        y      = star.y,
			        ox     = sprite:getWidth(),
			        oy     = sprite:getHeight()
			    }
				starLayer[i].name = star.name
				starLayer[i].properties = star.properties
				world:add(starLayer[i],star.x,star.y,star.width,star.height)

				i = i + 1
			end
		end
		map:removeLayer("star_spawn") 
	end

	-- SPAWN BOXES AND ADD TO COLLISION WORLD
	local spawnBoxes = function(objectName)
		local box
		local i = 1
		for k, object in pairs(map.objects) do
			if object.name == objectName then 
				box = object
				boxLayer[i] = Boxes(box.x,box.y,box.width,box.height)
				boxLayer[i].name = box.name
				boxLayer[i].properties = box.properties
				boxLayer[i].id = box.id
				world:add(boxLayer[i],box.x,box.y,box.width,box.height)

				i = i + 1
			end
		end
    	map:removeLayer("box_spawn")
	end

	addToBumpMap("plat")
	spawnPlayer("player")
	spawnEnemies("enemy1")
	addToBumpMap("enemy_bump_point")
	spawnStars("star_spawn")
	spawnBoxes("box_spawn")

	--ADD PATH TILES TO PATH TABLE
	pathLayer = map.layers["path"]
	path = {}
	for _, layer in ipairs(map.layers) do
		-- Entire layer
		if layer.name == "path" then
			for gid, tiles in pairs(map.tileInstances) do
				local tile = map.tiles[gid]
				local tileset = map.tilesets[tile.tileset]
				for _, instance in ipairs(tiles) do
					if instance.layer == layer then
						local object = {
						shape      = "rectangle",
						x          = instance.x,
						y          = instance.y,
						width      = tileset.tilewidth,
						height     = tileset.tileheight,
						properties = tile.properties
						}
						table.insert(path, object)
					end
				end
			end
		end
	end

	map:box2d_init(physicsWorld)

end

function Game:update(dt)
	local player = map.layers["PlayerSprite"].player
	map:update(dt)
	player:update(dt)
	-- for i=1,#map.layers["StarSprites"] do
	-- 	star = map.layers["StarSprites"][i]
	-- 	star:update(dt)
	-- end
	for i=1,#map.layers["EnemySprites"] do
		map.layers["EnemySprites"][i]:update(dt)
	end
	for i=1,#map.layers["BoxSprites"] do
		map.layers["BoxSprites"][i]:update(dt)
	end

	
end

function Game:draw()
	camera:set()

	camtx = camera.x
	camty = camera.y

    local player = map.layers["PlayerSprite"].player
    -- Translate world so that player is always centred
    if player.x > globalWidth/2 then
    	camtx = player.x - globalWidth / 2
    end
    if player.x < 0 then
    	camtx = player.x
    end
   	camty = player.y - globalHeight/2

   	--DRAW ENEMIES
	for i=1,#map.layers["EnemySprites"]do
		map.layers["EnemySprites"][i]:draw()
	end
	-- DRAW STARS
	for i=1,#map.layers["StarSprites"] do
		star = map.layers["StarSprites"][i]
		love.graphics.draw(
           	star.sprite,
           	math.floor(star.x + star.ox/2),
           	math.floor(star.y + star.oy/2),
           	0,
           	1,
           	1,
           	star.ox,
           	star.oy
       )
	end
	-- DRAW BOXES
	for i=1,#map.layers["BoxSprites"] do
		map.layers["BoxSprites"][i]:draw()
	end
				
	--DRAW PLAYER
	player:draw()
	--DRAW MAP
	map:draw(math.floor(-camtx), math.floor(-camty))
	-- -- SET CAMERA POSITION
	camera.x = math.floor(camtx)
	camera.y = math.floor(camty)

	-- PATH DEBUGGING
	if debugMode then
		for i = 1, #path do
			love.graphics.setColor(0, 1, 0)
			love.graphics.rectangle("line", path[i].x, path[i].y, path[i].width, path[i].height)
			love.graphics.setColor(1, 1, 1)
		end
	end

	camera:unset()
	
end
